<?php
namespace PhotoPrismUpload\API;

use Monolog\Logger;
use Monolog\Handler\HandlerInterface;

/** Simple factory to create a logger without needing to set stream handlers every time */
class LoggerFactory
{
    private static $handlers = [];

    /**
     * Add a new handler which is automatically added to the list of handlers
     *  for all _future_ loggers
     *
     * @param HandlerInterface $handler The handler to add
     *
     * @return void
     */
    public static function addHandler(HandlerInterface $handler): void
    {
        self::$handlers[] = $handler;
    }

    /**
     * Create a new Logger with the specified name
     *
     * @param string $name
     *
     * @return Logger
     */
    public static function create(string $name): Logger
    {
        $l = new Logger($name);
        $l->setHandlers(self::$handlers);
        return $l;
    }
}
